/**
 * Задатак 2 (јануар 2014)
 * Написати на програмском језику C програм који налази кориснике који нису на
 * време измирили рачуне за електричну енергију. У текстуалној датотеци
 * racuni.txt налазе се јединствени записи о рачунима корисника електричне
 * енергије, где сваки ред садржи целобројни идентификатор бројила и износ који
 * треба уплатити. У текстуалној датотеци uplate.txt налазе се подаци о уплатама
 * електричне енергије, при чему сваки ред садржи идентификатор бројила и износ
 * који је уплаћен. Ништа се не може претпоставити о броју и редоследу рачуна и
 * уплата у датотекама. Може се десити да за неке рачуне не постоји ниједна
 * уплата, а такође је могуће извршити произвољно много уплата за један рачун.
 * Све уплате односе се на постојеће идентификаторе бројила. Програм најпре
 * треба да учита податке о свим рачунима у једноструко уланчану листу, а затим
 * да чита једну по једну уплату из датотеке uplate.txt и ажурира износ дуга за
 * прочитани рачун. На крају програм треба да формира излазну текстуалну
 * датотека duznici.txt у којој сваки ред садржи идентификатор бројила и износ
 * дуга (негативан број). Водити рачуна о коректној употреби заузетих ресурса.
 *
 * Аутор: Игор Лукић
 */

#include <stdio.h>

typedef struct {
    int brojilo;
    float iznos;
} Racun;

typedef struct {
    int brojilo;
    float iznos;
} Uplata;

typedef struct elem {
    Racun racun;
    struct elem *sled;
} Elem;

Elem* dodaj(Elem* lista, Racun r) {
    Elem *novi, *tek;
    novi = malloc(sizeof(Elem));
    novi->racun = r;
    novi->sled = NULL;
    if (lista == NULL) {
        lista = novi;
    } else {
        for (tek = lista; tek->sled != NULL; tek = tek->sled);
        tek->sled = novi;
    }
}

Elem* obrisi(Elem *lista) {
    Elem *tek, *zaBrisanje;

    tek = lista;
    while (tek != NULL) {
        zaBrisanje = tek;
        tek = tek->sled;
        free(zaBrisanje);
    }

    return NULL;
}

Elem* ucitaj(FILE *dat) {
    Elem *lista = NULL;
    Racun racun;

    while (!feof(dat)) {
        fscanf(dat, "%d %f\n", &racun.brojilo, &racun.iznos);
        lista = dodaj(lista, racun);
    }

    return lista;
}

void izvrsiUplatu(Elem *lista, Uplata uplata) {
    Elem *tek;

    for (tek = lista; tek != NULL; tek = tek->sled) {
        if (tek->racun.brojilo == uplata.brojilo) {
            tek->racun.iznos -= uplata.iznos;
        }
    }
}

void ispis(Elem *lista, FILE *dat) {
    Elem *tek;
    for (tek = lista; tek != NULL; tek = tek->sled) {
        if (tek->racun.iznos > 0) {
            fprintf(dat, "%d %f\n", tek->racun.brojilo, tek->racun.iznos);
        }
    }
}

void main() {
    FILE *racuniDat, *uplateDat, *duzniciDat;
    Elem *lista = NULL;
    Uplata uplata;

    racuniDat = fopen("racuni.txt", "r");
    uplateDat = fopen("uplate.txt", "r");
    duzniciDat = fopen("duznici.txt", "w");

    if (racuniDat == NULL || uplateDat == NULL || duzniciDat == NULL) {
        printf("Otvaranje datoteka nije uspelo.\n");
        exit(1);
    }

    lista = ucitaj(racuniDat);

    while (!feof(uplateDat)) {
        fscanf(uplateDat, "%d %f\n", &uplata.brojilo, &uplata.iznos);
        izvrsiUplatu(lista, uplata);
    }

    ispis(lista, duzniciDat);

    lista = obrisi(lista);

    fclose(racuniDat);
    fclose(uplateDat);
    fclose(duzniciDat);
}